@@ -1,5 +1,5 @@
 /*
-    Copyright 2008-2012 Luigi Auriemma
+    Copyright 2008-2021 Luigi Auriemma
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include <sys/stat.h>
 #include <ctype.h>
 #include "mywav.h"
-//#include "resample2.c"
 
 #ifdef WIN32
 #else
@@ -44,7 +43,7 @@ typedef uint32_t    u32;
 
 
 
-#define VER         "0.2"
+#define VER         "0.2.1"
 
 enum {
     DOT = 1,
@@ -65,7 +64,7 @@ i16 *do_samples(FILE *fd, int wavsize, int *ret_samples, int bits);
 int do_mono(i16 *smp, int samples, int ch);
 void do_dcbias(i16 *smp, int samples);
 void do_normalize(i16 *smp, int samples);
-int do_reduce(i16 *smp, int samples, int *freq);
+int do_downsampling(i16 *smp, int samples, int *freq, int new_freq);
 void my_err(u8 *err);
 void std_err(void);
 
@@ -206,14 +205,16 @@ int main(int argc, char *argv[]) {
     if(fd != stdin) fclose(fd);
 
     samples = do_mono(smp, samples, fmt.wChannels);
+    fmt.wChannels        = 1;
+
     if(optimize) {
         do_dcbias(smp, samples);
         do_normalize(smp, samples);
     }
-    samples = do_reduce(smp, samples, &fmt.dwSamplesPerSec);
+
+    samples = do_downsampling(smp, samples, &fmt.dwSamplesPerSec, 8000);
 
     fmt.wFormatTag       = 0x0001;
-    fmt.wChannels        = 1;
     fmt.wBitsPerSample   = 16;
     fmt.wBlockAlign      = (fmt.wBitsPerSample >> 3) * fmt.wChannels;
     fmt.dwAvgBytesPerSec = fmt.dwSamplesPerSec * fmt.wBlockAlign;
@@ -810,21 +811,37 @@ void do_normalize(i16 *smp, int samples) {
 
 
 
-void *av_resample_init(int, int, int, int, int, double);
-int av_resample(void *, short *, short *, int *, int, int, int);
-void av_resample_close(void *);
+// code from https://programmer.help/blogs/5e87bf7ff41e0.html
+int resampleData(const int16_t *sourceData, int32_t sampleRate, uint32_t srcSize, int16_t *destinationData, int32_t newSampleRate) {
+    if (sampleRate == newSampleRate) {
+        if(destinationData != sourceData) {
+            memcpy(destinationData, sourceData, srcSize * sizeof(int16_t));
+        }
+        return srcSize;
+    }
+    if (sampleRate < newSampleRate) {
+        if(destinationData == sourceData) return srcSize;   // can't go forward
+    }
+    uint32_t last_pos = srcSize - 1;
+    uint32_t dstSize = (uint32_t) (srcSize * ((float) newSampleRate / sampleRate));
+    uint32_t idx;
+    for (idx = 0; idx < dstSize; idx++) {
+        float index = ((float) idx * sampleRate) / (newSampleRate);
+        uint32_t p1 = (uint32_t) index;
+        float coef = index - p1;
+        uint32_t p2 = (p1 == last_pos) ? last_pos : p1 + 1;
+        destinationData[idx] = (int16_t) ((1.0f - coef) * sourceData[p1] + coef * sourceData[p2]);
+    }
+    return idx;
+}
 
-int do_reduce(i16 *smp, int samples, int *freq) {
-    void    *res;
-    int     consumed,
-            newfreq     = 8000;
 
-    fprintf(stderr, "  resampling to  %dhz\n", newfreq);
-    res = av_resample_init(newfreq, *freq, 16, 10, 0, 0.8);
-    samples = av_resample(res, smp, smp, &consumed, samples, samples, 1);
-    av_resample_close(res);
 
-    *freq = newfreq;
+int do_downsampling(i16 *smp, int samples, int *freq, int new_freq) {
+    if(*freq <= new_freq) return(samples);
+    fprintf(stderr, "  resampling to: %dhz\n", new_freq);
+    samples = resampleData(smp, *freq, samples, smp, new_freq);
+    *freq = new_freq;
     return(samples);
 }
 
